@@ -47,6 +47,8 @@ typedef struct
 
 typedef struct
 {
+	gint priority;
+	gboolean preempt;
 	gchar *voice;
 	gchar *marker;
 	gchar *spelling;
@@ -145,6 +147,8 @@ void       	srs_speak_to_engine 	(SRSText 	*srs_text);
 int        	srs_shut_up    		(void);
 int        	srs_pause      		(void);
 int        	srs_resume     		(void);
+gboolean	srs_remove		(gint priority);
+void		srs_call_shut_up 	();
 /*____________________________< engine METHODS >______________________________*/
 
 #endif
