@@ -20,9 +20,11 @@
  */
 
 #include <stdio.h>
+#include <sys/types.h>
 #include <netinet/in.h>
 #include <netdb.h>
 #include <strings.h>
+#include <signal.h>
 
 #include <glib.h>
 #include "libsrconf.h"
@@ -61,7 +63,7 @@ extern gchar *fmt_for_xml (gchar *, gchar *, gchar *);
 
 gint 	brlmon_sd, 	/* BrlMonitor socket descriptor*/
 	remote_sd;	/* RemoteBraille socket descriptor*/
-
+gint 	src_brlmon_pid = -1;
 
 /* socket structures */
 #ifdef INET6
@@ -435,7 +437,7 @@ src_brlmon_terminate()
 	src_brlmon_quit ();
 	close (brlmon_sd);
 	brlmonsockstate = FALSE;
-	}
+    }
 }
 
 static void 
@@ -582,6 +584,33 @@ src_braille_show (gchar *message)
     g_free(text);
 }
 
+gboolean
+src_braille_translation_table_exist (const gchar *table)
+{
+    gchar *fn = NULL;
+    gboolean rv = FALSE;			
+    
+    /* derive the TT file name from the language name */
+    fn = g_strdup_printf ("%s.a2b", table);
+    /* test the file */
+    if (g_file_test (fn, G_FILE_TEST_EXISTS)) 
+    {
+	rv = TRUE;
+    }
+    else
+    {
+	gchar *dir_fn = NULL;
+	dir_fn = g_strconcat (BRAILLE_TRANS_TABLES_DIR,fn,NULL);
+	if (g_file_test (dir_fn, G_FILE_TEST_EXISTS)) 
+	    rv = TRUE;
+	g_free (dir_fn);
+    }
+    g_free (fn);
+    
+    return rv;
+}
+
+
 gboolean
 src_braille_restart ()
 {
@@ -604,6 +633,53 @@ src_braille_restart ()
 
 extern void brl_xml_input_proc (gchar *, gint);
 
+gboolean 
+brl_mon_run_process (void)
+{
+    if (g_file_test ("../brlmon/brlmonitor", G_FILE_TEST_EXISTS) &&
+        g_file_test ("../brlmon/brlmonitor", G_FILE_TEST_IS_EXECUTABLE) &&
+        g_file_test ("../brlmon/brlmonitor", G_FILE_TEST_IS_REGULAR))
+    {
+	gchar *args[] = {"../brlmon/brlmonitor", NULL};
+	if (!g_spawn_async ( ".", args , NULL , 
+			    G_SPAWN_DO_NOT_REAP_CHILD,
+			    NULL, NULL, &src_brlmon_pid, NULL))
+	{
+	    sru_message ("No \"%s\" binary file found.", 
+			    "brlmonitor");
+	    return FALSE;
+	}
+    }
+    else
+    if (g_file_test ("./brlmonitor", G_FILE_TEST_EXISTS) &&
+        g_file_test ("./brlmonitor", G_FILE_TEST_IS_EXECUTABLE) &&
+        g_file_test ("./brlmonitor", G_FILE_TEST_IS_REGULAR))
+    {
+	gchar *args[] = {"./brlmonitor", NULL};
+	if (!g_spawn_async ( ".", args , NULL , 
+			    G_SPAWN_DO_NOT_REAP_CHILD,
+			    NULL, NULL, &src_brlmon_pid, NULL))
+	{
+	    sru_message ("No \"%s\" binary file found.", 
+			    "brlmonitor");
+	    return FALSE;
+	}	
+    }
+    else
+    {
+	gchar *args[] = {"brlmonitor", NULL};
+	if (!g_spawn_async ( ".", args , NULL , 
+			    G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
+			    NULL, NULL, &src_brlmon_pid, NULL))
+	{
+	    sru_message ("No \"%s\" binary file found.",
+			    "brlmonitor");
+	    return FALSE;
+	}
+    }
+    return TRUE;
+}
+
 gboolean
 src_braille_monitor_init ()
 {
@@ -616,6 +692,13 @@ src_braille_monitor_init ()
 	    srconf_unset_key (SRCORE_BRAILLE_MONITOR_SENSITIVE, SRCORE_PATH);
 	    SET_SRCORE_CONFIG_DATA (SRCORE_BRAILLE_MONITOR_SENSITIVE, CFGT_BOOL, &brlmonsockstate);
 	}
+    if (!brl_mon_run_process ())
+	{
+	    fprintf (stderr, "**Can't initialize any module.\n");
+/*	    srconf_unset_key (SRCORE_BRAILLE_MONITOR_SENSITIVE, SRCORE_PATH);
+	    SET_SRCORE_CONFIG_DATA (SRCORE_BRAILLE_MONITOR_SENSITIVE, CFGT_BOOL, &brlmons);
+*/
+	}
     return TRUE;
 }
 
