@@ -37,6 +37,7 @@
 #include "srpres.h"
 #include "srintl.h"
 #include "screen-review.h"
+#include <libxml/parser.h>
 
 
 static SRLClientHandle src_srl_client;
@@ -61,6 +62,7 @@ extern gboolean panning;
 
 extern gint src_braille_offset;
 
+
 void
 src_present_null_sro ()
 {
@@ -122,14 +124,14 @@ src_present_real_sro (SRObject *obj)
 	    gchar offset[5];
 
 	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
-	    if (role == SR_ROLE_TEXT || role == SR_ROLE_TERMINAL)
+	    if (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL)
 	    {
 		SRLong offset;
 		SRTextAttribute *attr;
 		gchar status_[5] = "    ";
 
 		if (sro_text_get_caret_offset (obj, &offset, SR_INDEX_CONTAINER))
-		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"%s\"", offset + 4, src_braille_cursor_style);
+		    cursor = g_strdup_printf (" cursor=\"%ld\" cstyle=\"%s\"", offset, src_braille_cursor_style);
 	    
 		if (sro_text_get_text_attr_from_caret (obj, SR_TEXT_BOUNDARY_CHAR, 
 				    &attr, SR_INDEX_CONTAINER))
@@ -196,7 +198,7 @@ src_present_real_sro (SRObject *obj)
 	    SRObjectRoles role;
 	    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
 	    if (strcmp (reason, "object:text-changed:insert") == 0
-		    && (role == SR_ROLE_TEXT || role == SR_ROLE_TERMINAL))
+		    && (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL))
 	    {
 		gchar *diff;
 		if (sro_text_get_difference (obj, &diff, SR_INDEX_CONTAINER))
@@ -208,6 +210,10 @@ src_present_real_sro (SRObject *obj)
 			    g_free (src_last_key_echo);
 			    src_last_key_echo = NULL;
 			}
+			src_last_key_echo = diff;
+			src_cmd_queue_add ("kbd key");
+			src_cmd_queue_process ();
+			src_last_key_echo = NULL;
 		    }
 		    else
 		    {
@@ -233,7 +239,12 @@ src_present_real_sro (SRObject *obj)
 		txt = src_presentation_present_sro_for_device (obj, reason, "speech");
     		if (txt && txt[0])
     		{
-		    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE);
+		    SRObjectRoles role;
+		    sro_get_role (obj, &role, SR_INDEX_CONTAINER);
+		    if (role == SR_ROLE_STATUS_BAR && strcmp (reason , "focus:") == 0)
+			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, FALSE);
+		    else
+			src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 		}
 		g_free (txt);
 	    }
@@ -247,6 +258,11 @@ src_present_real_sro (SRObject *obj)
 	txt = src_presentation_present_sro_for_device (obj, reason, "magnifier");
 	if (txt)
 	{
+/*	    fprintf (stderr, "\n(%s %d) txt = %s",
+			__FILE__,
+			__LINE__,
+			txt);
+*/
 	    gchar *tmp;
 	    tmp = g_strconcat ("<MAGOUT><ZOOMER ID=\"generic_zoomer\" tracking=\"focus\" ",
 				txt,
@@ -297,7 +313,10 @@ src_present_crt_window ()
 	txt = src_presentation_present_sro_for_device (src_crt_window, reason, "speech");
     	if (txt && txt[0])
     	{
-	    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM);
+	    if (strcmp (reason, "window:create") == 0)
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_WARNING, FALSE);
+	    else
+		src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
 	}
 	g_free (txt);
     }
@@ -323,7 +342,7 @@ src_present_crt_tooltip ()
 	txt = src_presentation_present_sro_for_device (src_crt_tooltip, reason, "speech");
     	if (txt && txt[0])
     	{
-	    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM);
+	    src_speech_send_chunk (txt, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
 	}
 	g_free (txt);
     }
@@ -390,7 +409,7 @@ src_present_details ()
     }    
     
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     
     g_free (name);
     g_free (role);
@@ -430,7 +449,7 @@ src_present_layer_changed ()
 		
 	message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), tmp);
 	if (message && message[0])
-	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 
 	g_free (tmp);
 	g_free (message);
@@ -451,7 +470,7 @@ src_present_layer_timeout ()
 	
 	message = src_xml_make_part ("TEXT", src_speech_get_voice ("system"), tmp);
 	if (message && message[0])
-	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	    src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
 
 	g_free (tmp);
 	g_free (message);
@@ -639,11 +658,23 @@ src_process_config_changed_for_braille (SRConfigStructure *config)
 	{
 	    if (!strcmp (config->newvalue, src_braille_translation_table))
 		return;
-	    g_free (src_braille_translation_table);
-		
-	    src_braille_translation_table = g_strdup (config->newvalue);				    
 	    
-	    restart = TRUE;
+	    if (src_braille_translation_table_exist (config->newvalue))
+	    {
+		g_free (src_braille_translation_table);
+		
+		src_braille_translation_table = g_strdup (config->newvalue);				    
+		
+		restart = TRUE;
+	    }
+	    else
+	    {
+		gchar *value =  DEFAULT_BRAILLE_TRANSLATION;
+		
+		srconf_set_config_data (BRAILLE_TRANSLATION, CFGT_STRING, value, CFGM_BRAILLE);		
+		
+		restart = FALSE;
+	    }	    
 	}
 	else
 	{
@@ -969,7 +1000,7 @@ static gboolean src_crosswire	    	= FALSE;
     }
     else
     if (config->key && 
-	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_CURSOR_MAG) == 0)
+	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_CURSOR_SCALE) == 0)
     {
 	if (src_use_magnifier && 
 	    config->newvalue &&
@@ -1590,7 +1621,56 @@ static gboolean src_crosswire	    	= FALSE;
 		src_magnifier_stop_panning (0);
 	}
     }
+    else
+    if (config->key && 
+	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_ALIGNMENT_X) == 0)
+    {
+    	if (src_use_magnifier && 
+	    config->newvalue &&
+	    config->type == CFGT_STRING);
+	{
+	    gchar *alignment_x = (gchar*)config->newvalue;
+	
+	    if (src_use_magnifier) 
+	    {
+		gchar *mag_out = NULL;
+		
+		mag_out = g_strdup_printf("<MAGOUT><ZOOMER ID=\"generic_zoomer\"  AlignmentX=\"%s\"></ZOOMER></MAGOUT>", alignment_x);
 
+		if (mag_out)
+		{
+		    src_magnifier_send (mag_out);
+		    g_free (mag_out);
+		    mag_out = NULL;
+		}
+	    }
+	}
+    }
+    else
+    if (config->key && 
+	strcmp (config->key, MAGNIFIER_ACTIVE_SCHEMA MAGNIFIER_ALIGNMENT_Y) == 0)
+    {
+    	if (src_use_magnifier && 
+	    config->newvalue &&
+	    config->type == CFGT_STRING);
+	{
+	    gchar *alignment_y = (gchar*)config->newvalue;
+	
+	    if (src_use_magnifier) 
+	    {
+		gchar *mag_out = NULL;
+		
+		mag_out = g_strdup_printf ("<MAGOUT><ZOOMER ID=\"generic_zoomer\"  AlignmentY=\"%s\"></ZOOMER></MAGOUT>", alignment_y);
+
+		if (mag_out)
+		{
+		    src_magnifier_send (mag_out);
+		    g_free (mag_out);
+		    mag_out = NULL;
+		}		
+	    }
+	}
+    }
 }
 
 static void
@@ -1605,13 +1685,27 @@ src_process_config_changed_for_control (SRConfigStructure *config)
     /*src_update_key ();*/
 }
 
+static void
+src_process_change_for_presentation (gboolean first)
+{
+    gchar *name = "default";
+    gchar *name2;
+	
+    srconf_get_data_with_default (PRESENTATION_ACTIVE_SETTING, CFGT_STRING, &name, &name, PRESENTATION_PATH);
+
+    name2 = g_strconcat (PRESENTATION_DIR, name, ".xml", NULL); 
+    if (!first)
+	src_presentation_terminate ();
+    src_presentation_init (name2);
+    g_free (name2);
+}
+
 static void
 src_process_config_changed_for_presentation (SRConfigStructure *config)
 {
     if (config)
     {
-	src_presentation_terminate ();
-	src_presentation_init ();
+	src_process_change_for_presentation (FALSE);
     }
 }
 
@@ -1737,7 +1831,7 @@ src_kb_key_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_key_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_MESSAGE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_MESSAGE, TRUE);
     g_free (message); 
     
     return TRUE;    
@@ -1753,7 +1847,7 @@ src_kb_punct_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_punct_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message); 
     
     return TRUE;    
@@ -1769,7 +1863,7 @@ src_kb_space_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_space_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message); 
     
     return TRUE;    
@@ -1785,7 +1879,7 @@ src_kb_modifier_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_modifier_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_IDLE, TRUE);
     g_free (message); 
     
     return TRUE;    
@@ -1802,7 +1896,7 @@ src_kb_cursor_echo ()
 	return FALSE;
     message = src_xml_format ("TEXT", src_speech_get_voice ("system"), src_last_cursor_echo);
     if (message && message[0])
-	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_SYSTEM);
+	src_speech_send_chunk (message, SRC_SPEECH_PRIORITY_SYSTEM, TRUE);
     g_free (message); 
 
     return TRUE;    
@@ -2021,7 +2115,10 @@ src_process_key (SREvent *event,
 
     if (!srhotkey_data->keystring)
 	return;
-    
+    if (strcmp (srhotkey_data->keystring, "Control_L") == 0)
+	if (src_use_speech)
+	    src_speech_shutup ();
+
     if (strncmp (srhotkey_data->keystring, "Alt", 3) == 0 ||
 	strncmp (srhotkey_data->keystring, "Control", 7) == 0 ||
 	strncmp (srhotkey_data->keystring, "Shift", 5) == 0 ||
@@ -2371,6 +2468,11 @@ src_get_defaults ()
 	gchar *default_braille_transaltion_table = DEFAULT_BRAILLE_TRANSLATION;
 	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_TRANSLATION, CFGT_STRING, 
 		&src_braille_translation_table, default_braille_transaltion_table);
+		
+	if (!src_braille_translation_table_exist (src_braille_translation_table))
+	{
+	    srconf_set_config_data (BRAILLE_TRANSLATION, CFGT_STRING, DEFAULT_BRAILLE_TRANSLATION, CFGM_BRAILLE);
+	}	    
     }
     
     {
@@ -2401,24 +2503,29 @@ src_get_defaults ()
     
     if ((gint)src_use_braille == -1)
     {
+	src_use_braille = DEFAULT_SRCORE_BRAILLE_ACTIVE;
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_ACTIVE, CFGT_BOOL, 
 	    &src_use_braille, &src_use_braille);
     }
     
+    
     if ((gint)src_use_braille_monitor == -1)
     {
+	src_use_braille_monitor = DEFAULT_SRCORE_BRAILLE_MONITOR_ACTIVE;
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_MONITOR_ACTIVE, CFGT_BOOL, 
 	    &src_use_braille_monitor, &src_use_braille_monitor);
     }
     
     if ((gint)src_use_speech == -1)
     {
+	src_use_speech = DEFAULT_SRCORE_SPEECH_ACTIVE;
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_SPEECH_ACTIVE, CFGT_BOOL, 
 					    &src_use_speech, &src_use_speech);
     }
 
     if ((gint)src_use_magnifier == -1)
     {
+	src_use_magnifier = DEFAULT_SRCORE_MAGNIF_ACTIVE;
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_MAGNIF_ACTIVE, CFGT_BOOL, 
 					    &src_use_magnifier, &src_use_magnifier);
     }
@@ -2457,9 +2564,95 @@ src_present_last_message ()
     return TRUE;
 }
 
+void
+src_set_command_lines (const gchar *mouse_tracking,
+		       const gchar *focus_tracking,
+		       gboolean    invert,
+		       gint 	   zoomfactor)
+{
+    if (mouse_tracking)
+    {
+    	gchar *mt = g_ascii_strdown (mouse_tracking, -1);
+	if (!strcmp (mt, "centered"))
+	{
+	    g_free (mt);
+	    mt = g_strdup ("mouse-centered");
+	}
+	else if (!strcmp (mt, "proportional"))
+	{
+	    g_free (mt);
+	    mt = g_strdup ("mouse-proportional");
+	}
+	else if (!strcmp (mt, "push"))
+	{
+	    g_free (mt);
+	    mt = g_strdup ("mouse-push");
+	}
+	else if (!strcmp (mt, "none"))
+	{
+	    g_free (mt);
+	    mt = g_strdup ("none");
+	}
+	else
+	{
+	    g_free (mt);
+	    mt = NULL;
+	}
+	if (mt)
+	{
+	    srconf_set_data (MAGNIFIER_MOUSE_TRACKING,
+			  CFGT_STRING, 
+			  mt,
+			  MAGNIFIER_CONFIG_PATH);
+	
+	}
+	g_free (mt);
+    }
+    
+    if (focus_tracking)
+    {
+	gchar *ft = g_ascii_strdown (focus_tracking, -1);
+	if (!strcmp (ft, "none") ||
+	    !strcmp (ft, "centered") ||
+	    !strcmp (ft, "auto"))
+	{
+	    srconf_set_data (MAGNIFIER_ALIGNMENT_Y,
+			  CFGT_STRING, 
+			  ft,
+			  MAGNIFIER_CONFIG_PATH);
+    
+	    srconf_set_data (MAGNIFIER_ALIGNMENT_X,
+			  CFGT_STRING, 
+			  ft,
+			  MAGNIFIER_CONFIG_PATH);
+	}
+	g_free (ft);
+    }
+    
+    srconf_set_data (MAGNIFIER_INVERT,
+			  CFGT_BOOL, 
+			  &invert,
+			  MAGNIFIER_CONFIG_PATH);
+    
+    if (0 < zoomfactor && 17 > zoomfactor)
+    {
+    	srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
+			  CFGT_INT, 
+			  &zoomfactor,
+			  MAGNIFIER_CONFIG_PATH);
+    	srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
+			  CFGT_INT, 
+			  &zoomfactor,
+			  MAGNIFIER_CONFIG_PATH);
+    }
+}    
 
 static void
-src_init()
+src_init(const gchar *mouse_tracking,
+	 const gchar *focus_tracking,
+	 const gchar *config_source,
+	 gboolean    invert,
+	 gint 	     zoomfactor)
 {
     SRLClient client;
     gchar *message = _("Welcome to Gnopernicus");
@@ -2470,14 +2663,14 @@ src_init()
     sru_init();
 
     /* initialize SRConf */
-    srconf_init((SRConfCB)src_event_sink, "/apps/gnopernicus");
+    srconf_init((SRConfCB)src_event_sink, "/apps/gnopernicus", config_source);
 
     /* layout init */
     src_ctrl_init ();
     
 /*    src_speech_count_mode = DEFAULT_SPEECH_REPEAT_TYPE;*/
     src_get_defaults ();
-
+    
     /* initialize Braille */
     if (src_use_braille)
     {
@@ -2492,6 +2685,12 @@ src_init()
     if (src_use_speech)
 	src_use_speech = src_speech_init ();
 
+    /* set the command lines arguments in gconf */
+    src_set_command_lines (mouse_tracking,
+		    	    focus_tracking,
+			    invert,
+			    zoomfactor);
+
     /* initialize Magnifier */
     if (src_use_magnifier)
     {
@@ -2500,7 +2699,8 @@ src_init()
 	    src_magnifier_create ();
     }
     
-    src_presentation_init ();	
+    src_process_change_for_presentation (TRUE);
+
     /* initialize SRLow */
     srl_init();
 
@@ -2593,8 +2793,7 @@ src_terminate()
     /* terminate Speech */
     if (src_use_speech)
 	src_speech_terminate ();
-
-
+	
     /* terminate SRConf */
     SET_SRCORE_CONFIG_DATA(SRCORE_EXIT_ACK_KEY, CFGT_BOOL, &exitack);
     exitack = !exitack;
@@ -2620,7 +2819,7 @@ src_terminate()
     if (src_last_cursor_echo)
 	g_free (src_last_cursor_echo);
 }
-
+    
 gint
 main (gint argc, 
       gchar **argv)
@@ -2633,6 +2832,14 @@ main (gint argc,
     gboolean disable_magnifier	= FALSE;
     gboolean enable_braille_monitor 	= FALSE;
     gboolean disable_braille_monitor 	= FALSE;    
+    
+    gchar 	*config_source 	= NULL;
+    gchar 	*mouse_tracking = NULL;
+    gchar	*focus_tracking = NULL;
+    gint  	zoom_factor     = 0;
+    gboolean 	invert 	    	= FALSE;
+
+    
 
     struct poptOption poptopt[] = 
     {		
@@ -2642,16 +2849,16 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &enable_braille,		
 	 0, 
-	 "enable braille service", 
+	 "Enable braille service", 
 	 NULL
 	},
 	{
 	 "disable-braille",	
-	 'r', 	
+	 'B', 	
 	 POPT_ARG_NONE, 
 	 &disable_braille,		
 	 0, 
-	 "enable braille service", 
+	 "Disable braille service", 
 	 NULL
 	},
 	{
@@ -2660,16 +2867,16 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &enable_speech,		
 	 0, 
-	 "enable speech service", 
+	 "Enable speech service", 
 	 NULL
 	},
 	{
 	 "disable-speech",	
-	 'e', 	
+	 'S', 	
 	 POPT_ARG_NONE, 
 	 &disable_speech,		
 	 0, 
-	 "disable speech service", 
+	 "Disable speech service", 
 	 NULL
 	},
 	{
@@ -2678,16 +2885,16 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &enable_magnifier,		
 	 0, 
-	 "enable magnifier service",  
+	 "Enable magnifier service",  
 	 NULL
 	},
 	{
 	 "disable-magnifier",	
-	 'a', 	
+	 'M', 	
 	 POPT_ARG_NONE, 
 	 &disable_magnifier,		
 	 0, 
-	 "disable magnifier service",  
+	 "Disable magnifier service",  
 	 NULL
 	},
 
@@ -2697,16 +2904,16 @@ main (gint argc,
 	 POPT_ARG_NONE, 
 	 &enable_braille_monitor, 	
 	 0, 
-	 "enable braille monitor service", 
+	 "Enable braille monitor service", 
 	 NULL
 	},
 	{
 	 "disable-braille-monitor", 
-	 'n', 
+	 'O', 
 	 POPT_ARG_NONE, 
 	 &disable_braille_monitor, 	
 	 0, 
-	 "disable braille monitor service", 
+	 "Disable braille monitor service", 
 	 NULL
 	},
 	{
@@ -2727,6 +2934,53 @@ main (gint argc,
 	 "Braille Device", 
 	 "DEVICE NAME"
 	},
+	{
+	"settings-dir", 
+	 'c', 
+	 POPT_ARG_STRING, 
+	 &config_source, 	
+	 0, 
+	 "Use gconf settings file", 
+	 "PATH"
+	},
+	{
+	"magnification-zoom", 
+	 'z', 
+	 POPT_ARG_INT, 
+	 &zoom_factor, 	
+	 0, 
+	 "Set magnification zoom level", 
+	 "[1..16]"
+	},
+	{
+	"magnification-invert", 
+	 'i', 
+	 POPT_ARG_NONE, 
+	 &invert, 	
+	 0, 
+	 "Magnification with reverse video", 
+	 ""
+	},
+
+	{
+	"magnification-mouse-tracking", 
+	 't', 
+	 POPT_ARG_STRING, 
+	 &mouse_tracking, 	
+	 0, 
+	 "How the mouse pushed the region", 
+	 "[CENTERED, PROPORTION, PUSH, NONE]"
+	},
+
+	{
+	"magnification-focus-tracking", 
+	 'f', 
+	 POPT_ARG_STRING, 
+	 &focus_tracking, 	
+	 0, 
+	 "How the focus pushed the region", 
+	 "[AUTO, CENTERED, NONE]"
+	},
 	{
 	 NULL, 0,     0, NULL, 0
 	}
@@ -2771,7 +3025,21 @@ main (gint argc,
     src_use_magnifier = (disable_magnifier ? FALSE :
 			(enable_magnifier  ? TRUE : -1));
 
-    src_init ();
+    if (src_braille_device)
+    {
+	gchar *tmp = NULL;
+	tmp = g_utf8_strup (src_braille_device, -1);
+	g_free (src_braille_device);
+	src_braille_device = tmp;
+    }
+        
+
+    src_init (mouse_tracking, focus_tracking, config_source, invert, zoom_factor);
+    
+    g_free (mouse_tracking);
+    g_free (focus_tracking);
+    g_free (config_source);
+
 
     sru_entry_loop ();
 
