@@ -1659,9 +1659,23 @@ srl_set_watch_for_object (SRObject *obj)
 
     srl_assert (obj);
 
+    srl_unwatch_all_objects ();
     acc = sro_get_acc (obj);
     Accessible_ref (acc);
     g_ptr_array_add (srl_watched_acc, acc);
 
     return TRUE;
 }
+
+void
+srl_unwatch_all_objects ()
+{
+    gint i;
+    
+    for (i = 0; i < srl_watched_acc->len; i++)
+    {
+	Accessible *acc = g_ptr_array_index (srl_watched_acc, i);
+	Accessible_unref (acc);
+    }
+    g_ptr_array_set_size (srl_watched_acc, 0);
+}
